package info.msxlaunchers.openmsx.launcher.data.game;

import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.data.game.constants.Genre;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

public class GameTest
{
	@Test
	public void testCompleteGameConstruction()
	{
		Game game = Game.name( "name" ).info( "info" ).machine( "machine").romA( "romA" ).romB( "romB" )
			.extensionRom( "extensionRom" ).diskA( "diskA" ).diskB( "diskB" ).tape( "tape" )
			.harddisk( "harddisk" ).laserdisc( "laserdisc" )
			.isMSX( true ).isMSX2( true ).isMSX2Plus( true ).isTurboR( true )
			.isPSG( true ).isSCC( true ).isSCCI( true ).isPCM( true )
			.isMSXMUSIC( true ).isMSXAUDIO( true ).isMoonsound( true ).isMIDI( true )
			.genre1( Genre.ACTION ).genre2( Genre.ADVENTURE_TEXT_AND_GFX )
			.msxGenID( 80 ).sha1Code( "123456789abcdef" ).size( 16 ).screenshotSuffix( "suffix" )
			.tclScript( "tclScript" ).build();

		assertEquals( "name", game.getName() );
		assertEquals( "info", game.getInfo() );
		assertEquals( "machine", game.getMachine() );
		assertEquals( "romA", game.getRomA() );
		assertEquals( "romB", game.getRomB() );
		assertEquals( "extensionRom", game.getExtensionRom() );
		assertEquals( "diskA", game.getDiskA() );
		assertEquals( "diskB", game.getDiskB() );
		assertEquals( "tape", game.getTape() );
		assertEquals( "harddisk", game.getHarddisk() );
		assertEquals( "laserdisc", game.getLaserdisc() );
		assertTrue( game.isMSX() );
		assertTrue( game.isMSX2() );
		assertTrue( game.isMSX2Plus() );
		assertTrue( game.isTurboR() );
		assertTrue( game.isPSG() );
		assertTrue( game.isSCC() );
		assertTrue( game.isSCCI() );
		assertTrue( game.isPCM() );
		assertTrue( game.isMSXMUSIC() );
		assertTrue( game.isMSXAUDIO() );
		assertTrue( game.isMoonsound() );
		assertTrue( game.isMIDI() );
		assertEquals( Genre.ACTION, game.getGenre1() );
		assertEquals( Genre.ADVENTURE_TEXT_AND_GFX, game.getGenre2() );
		assertEquals( 80, game.getMsxGenID() );
		assertEquals( "123456789abcdef", game.getSha1Code() );
		assertEquals( 16, game.getSize() );
		assertEquals( "suffix", game.getScreenshotSuffix() );
		assertEquals( "tclScript", game.getTclScript() );
	}

	@Test( expected = IllegalArgumentException.class )
	public void testMustHaveMinumumFields()
	{
		Game.info( "info" ).machine( "machine")
				.isMSX( true ).isMSX2( true ).isMSX2Plus( true ).isTurboR( true )
				.isPSG( true ).isSCC( true ).isSCCI( true ).isPCM( true )
				.isMSXMUSIC( true ).isMSXAUDIO( true ).isMoonsound( true ).isMIDI( true )
				.genre1( Genre.ACTION ).genre2( Genre.ADVENTURE_TEXT_AND_GFX )
				.msxGenID( 80 ).sha1Code( "123456789abcdef" ).size( 16 ).screenshotSuffix( "suffix" )
				.build();
	}

	@Test
	public void testEqualityAndHashcode()
	{
		Game game1 = Game.name( "sameName" ).romA( "romA1" ).build();
		Game game2 = Game.name( "sameName" ).romA( "romA2" ).build();
		Game game3 = Game.name( "differentName" ).romA( "romA" ).build();
		Game game4 = Game.romA( "romA" ).build();

		assertEquals( game1, game2 );
		assertEquals( game1.hashCode(), game2.hashCode() );
		assertEquals( game1, game1 );
		assertEquals( game1.hashCode(), game1.hashCode() );
		assertEquals( game2, game2 );
		assertEquals( game2.hashCode(), game2.hashCode() );
		assertNotEquals( game1, game3 );
		assertNotEquals( game1.hashCode(), game3.hashCode() );
		assertNotEquals( game2, game3 );
		assertNotEquals( game2.hashCode(), game3.hashCode() );
		assertNotEquals( game4, game3 );
		assertNotEquals( game4.hashCode(), game3.hashCode() );
		assertNotEquals( game1, null );

		//in case of name==null, objects will not be equal but their hash code is the default.
		//TODO: review
		assertNotEquals( game4, game4 );
		assertEquals( game4.hashCode(), game4.hashCode() );
	}

	@Test
	public void testMediaType()
	{
		Game game = Game.name( "name" ).build();

		assertFalse( game.isROM() );
		assertFalse( game.isDisk() );
		assertFalse( game.isTape() );
		assertFalse( game.isHarddisk() );
		assertFalse( game.isLaserdisc() );

		Game gameROM = Game.romA( "romFile" ).build();

		assertTrue( gameROM.isROM() );
		assertFalse( gameROM.isDisk() );
		assertFalse( gameROM.isTape() );
		assertFalse( gameROM.isHarddisk() );
		assertFalse( gameROM.isLaserdisc() );

		Game gameDisk = Game.diskA( "diskFile" ).build();

		assertFalse( gameDisk.isROM() );
		assertTrue( gameDisk.isDisk() );
		assertFalse( gameDisk.isTape() );
		assertFalse( gameDisk.isHarddisk() );
		assertFalse( gameDisk.isLaserdisc() );

		Game gameTape = Game.tape( "tapeFile" ).build();

		assertFalse( gameTape.isROM() );
		assertFalse( gameTape.isDisk() );
		assertTrue( gameTape.isTape() );
		assertFalse( gameTape.isHarddisk() );
		assertFalse( gameTape.isLaserdisc() );

		Game gameHarddisk = Game.harddisk( "harddiskFile" ).build();

		assertFalse( gameHarddisk.isROM() );
		assertFalse( gameHarddisk.isDisk() );
		assertFalse( gameHarddisk.isTape() );
		assertTrue( gameHarddisk.isHarddisk() );
		assertFalse( gameHarddisk.isLaserdisc() );

		Game gameLaserdisk = Game.laserdisc( "laserdiscFile" ).build();

		assertFalse( gameLaserdisk.isROM() );
		assertFalse( gameLaserdisk.isDisk() );
		assertFalse( gameLaserdisk.isTape() );
		assertFalse( gameLaserdisk.isHarddisk() );
		assertTrue( gameLaserdisk.isLaserdisc() );
	}

	@Test
	public void testResetOfSomeFieldsIfEmpty()
	{
		//need to test that all String fields get set to null if they are empty Strings
		
		//for test 1, need to set at least the name field to be non-empty because we cannot have all fields
		//be empty
		Game game = Game.machine( "" ).name( "name" ).info( "" ).romA( "" ).romB( "" ).extensionRom( "" )
				.diskA( "" ).diskB( "" ).tape( "" ).harddisk( "" ).laserdisc( "" )
				.sha1Code( "" ).tclScript( "" ).build();

		assertNull( game.getMachine() );
		assertNull( game.getInfo() );
		assertNull( game.getRomA() );
		assertNull( game.getRomB() );
		assertNull( game.getExtensionRom() );
		assertNull( game.getDiskA() );
		assertNull( game.getDiskB() );
		assertNull( game.getTape() );
		assertNull( game.getHarddisk() );
		assertNull( game.getLaserdisc() );
		assertNull( game.getSha1Code() );
		assertNull( game.getTclScript() );

		//now make sure name fields also gets set to null if emoty String
		game = Game.name( "" ).romA( "romA" ).build();

		assertNull( game.getName() );
	}
}