/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.view.swing.component;

import info.msxlaunchers.openmsx.launcher.ui.view.swing.MainWindow;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

/**
 * @since v1.1
 * @author Sam Elsharif
 *
 */
@SuppressWarnings("serial")
public class ConfirmationWindow extends JDialog implements ActionListener
{
	private final Component parent;
	private final String title;
	private final String message;
	private final String[] buttonLabels;
	private final boolean rightToLeft;

	private JButton[] buttons;
	private int result;

	public ConfirmationWindow(Component parent, String title, String message, String[] buttonLabels, boolean rightToLeft)
	{
		this.parent = parent;
		this.title = title;
		this.message = message;
		this.buttonLabels = buttonLabels;
		this.rightToLeft = rightToLeft;

		buttons = new JButton[buttonLabels.length];

		//when clicking the close button, return a generic -1, which should be interpreted as cancel
		addWindowListener(new WindowAdapter()
		{
			public void windowClosed(WindowEvent e)
			{
				result = -1;
			}

			public void windowClosing(WindowEvent e)
			{
				result = -1;
			}
		});
	}

	public int displayAndGetResult()
	{
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setTitle(title);
		setModalityType(ModalityType.APPLICATION_MODAL);
		setResizable(false);
		setBounds(0, 0, 590, 140);
		getContentPane().setLayout(new BorderLayout());
		{
			JPanel messagePane = new JPanel();
			getContentPane().add(messagePane, BorderLayout.NORTH);
			{
				JLabel messageLabel = new JLabel(message, UIManager.getIcon("OptionPane.warningIcon"), SwingConstants.TRAILING);
				if(rightToLeft)
				{
					messageLabel.setHorizontalAlignment(SwingConstants.RIGHT);
					messageLabel.setHorizontalTextPosition(SwingConstants.LEFT);
				}
				else
				{
					messageLabel.setHorizontalAlignment(SwingConstants.LEFT);
					messageLabel.setHorizontalTextPosition(SwingConstants.RIGHT);
				}
				messageLabel.setPreferredSize(new Dimension(550,50));
				messagePane.add(messageLabel);
			}

			JPanel buttonsPane = new JPanel();
			getContentPane().add(buttonsPane, BorderLayout.SOUTH);
			buttonsPane.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
			{
				for(int counter = 0; counter < buttons.length; counter++)
				{
					buttons[counter] = new JButton(buttonLabels[counter]);
					buttons[counter].addActionListener(this);
					buttons[counter].setPreferredSize(MainWindow.BUTTON_DIMENSION);
					buttonsPane.add(buttons[counter]);
				}
			}
			if(rightToLeft)
			{
				buttonsPane.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
			}
		}

        setLocationRelativeTo(parent);
		setVisible(true);

		return result;
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		for(int counter = 0; counter < buttons.length; counter++)
		{
			if(source == buttons[counter])
			{
				result = counter;
				dispose();
				break;
			}
		}
	}
}
